<?php
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Slim\Factory\AppFactory;
use Selective\BasePath\BasePathMiddleware;

require __DIR__ . '/../vendor/autoload.php';
require __DIR__ . '/../config/db.php';

header('Access-Control-Allow-Origin: *');
header('Access-Control-Allow-Methods: *');
header('Access-Control-Allow-Headers: Origin, X-Requested-With, Content-Type, Accept, Authorization');
header("Access-Control-Allow-Credentials: true");
header('Content-Type: application/json');


$app = AppFactory::create();

$app->addRoutingMiddleware();
$app->add(new BasePathMiddleware($app));
$app->addErrorMiddleware(true, true, true);
$app->addBodyParsingMiddleware();


$app->get('/hello', function (Request $request, Response $response, $args) {

    $response->getBody()->write("Hello world!");
    return $response;
});


$app->get('/', function (Request $request, Response $response, $args) {

    $response->getBody()->write("Hello world!");
    return $response;
});

$app->post('/signin', function (Request $request, Response $response, array $args) {

    $allPostParams = $request->getParsedBody();

    if(!isset($allPostParams['code'])) {
        $error = array(
            "code" => "error",
            "message" => "missing params"
        );
        $response->getBody()->write(json_encode($error));
        return $response->withHeader('content-type', 'application/json')->withStatus(401);
    }

    $code = $allPostParams['code'];
    $sql = "SELECT clients.*, customs.primary_color, customs.secondary_color, customs.logo FROM clients INNER JOIN codes ON clients.ID = codes.client_id AND codes.code = '".strval($code)."' LEFT JOIN customs ON clients.ID = customs.client_id LIMIT 1";
    try {
        $db = new Database();
        $conn = $db->connect();
        $stmt = $conn->query($sql);

        $result = $stmt->fetch();
        $db = null;

        /* Vérifier si le resultat est OK */

        if($result) {

            $colors[] = [
                "variable" => "--primary",
                "value" => $result["primary_color"],
            ];

            $colors[] = [
                "variable" => "--secondary",
                "value" => $result["secondary_color"],
            ];
            $logo = $result["logo"];
            $etab_id = $result["ID"];
            $name = $result["name"];
            $url =  $result["path"];
            $urlEndPoint = $url . "/jwt-auth/v1/token";
            //
            $data = array('username' => 'interlude', 'password' => 'toto');

            $options = array(
                'http' => array(
                    'header'  => "Content-type: application/x-www-form-urlencoded\r\n",
                    'method'  => 'POST',
                    'content' => http_build_query($data)
                )
            );
            $context  = stream_context_create($options);
            $resultJWT = @file_get_contents($urlEndPoint, false, $context);
            
            if ($resultJWT === FALSE) { 
                $dataToSendBack = array(
                    "code" => "error",
                    "message" => "Impossible de contacter le serveur - vérifier la configuration"
                );
                $response->getBody()->write(json_encode($dataToSendBack));
                return $response->withHeader('content-type', 'application/json')->withStatus(401);
            } 
             
            $resultJSON = json_decode($resultJWT);
            $token = $resultJSON->data->token;
            
           
            $dataToSendBack = array(
                "code" => "success",
                "data" => array(
                    "token" => $token,
                    "etab_id" => $etab_id,
                    "url" => $url,
                    "name" => $name,
                    "colors" => $colors,
                    "logo" => $logo,
                ),
            );

             $response->getBody()->write(json_encode($dataToSendBack));
             return $response->withHeader('content-type', 'application/json')->withStatus(200);

        } else  {

            $dataToSendBack = array(
                "code" => "error",
                "message" => "Identifiant incorrect"
            );

            $response->getBody()->write(json_encode($dataToSendBack));
                return $response->withHeader('content-type', 'application/json')->withStatus(401);
        }

    } catch (PDOException $e) {
        $error = array(
            "code" => "error",
            "message" => $e->getMessage()
        );
        $response->getBody()->write(json_encode($error));
        return $response->withHeader('content-type', 'application/json')->withStatus(500);
    }

});

$app->post('/code', function (Request $request, Response $response, array $args) {



    $allPostParams = $request->getParsedBody();
   
    if(!isset($allPostParams['codes']) || !isset($allPostParams['ID'])) {
        $error = array(
            "code" => "error",
            "message" => "missing params"
        );
        $response->getBody()->write(json_encode($error));
        return $response->withHeader('content-type', 'application/json')->withStatus(401);
    }

    $codes = $allPostParams['codes'];    
    $ID = $allPostParams['ID'];    

    $backup = null;

    if(isset($ID)) {
        try {
            $db = new Database();
            $conn = $db->connect();   

            $stmt_select = $conn->prepare("SELECT code FROM `codes` WHERE client_id = :ID");
            $stmt_select->bindParam(':ID', $ID, PDO::PARAM_STR);
            $stmt_select->execute();
            $backup = $stmt_select->fetchAll(PDO::FETCH_COLUMN, 0);
            
            $stmt_delete = $conn->prepare("DELETE FROM `codes` WHERE client_id = :ID");
            $stmt_delete->bindParam(':ID', $ID, PDO::PARAM_STR);
            $stmt_delete->execute();
       
            $stmt_insert = $conn->prepare("INSERT INTO `codes`(`code`, `client_id`) VALUES (:code, :client_id)");

            if(count($codes) > 0) {
                foreach($codes as $code) {
                
                    $stmt_insert->bindParam(':code', $code, PDO::PARAM_STR);
                    $stmt_insert->bindParam(':client_id', $ID, PDO::PARAM_STR);
                    $stmt_insert->execute();
    
                }
            } 

            $dataToSendBack = array(
                "code" => "success",
                "message" => "Codes mis à jour"
            );
            
            $response->getBody()->write(json_encode($dataToSendBack));
            return $response->withHeader('content-type', 'application/json')->withStatus(200);

           
        } catch (PDOException $e) {

            if($backup) {

                $stmt_insert_backup = $conn->prepare("INSERT IGNORE INTO `codes`(`code`, `client_id`) VALUES (:code, :client_id)");

                if(count($backup) > 0) {
                    foreach($backup as $code) {
                        $stmt_insert_backup->bindParam(':code', $code, PDO::PARAM_STR);
                        $stmt_insert_backup->bindParam(':client_id', $ID, PDO::PARAM_STR);
                        $stmt_insert_backup->execute();
                    }
                } 
               
                $error = array(
                    "code" => "error",
                    "message" => "Mot de passe déja utilisé par une autre application ulliS",
                );
                $response->getBody()->write(json_encode($error));
                return $response->withHeader('content-type', 'application/json')->withStatus(200);
            }

            $error = array(
                "code" => "error",
                "message" => $e->getMessage()
            );
            $response->getBody()->write(json_encode($error));
            return $response->withHeader('content-type', 'application/json')->withStatus(200);
        }

    } else {
        $error = array(
            "code" => "error",
            "message" => "Missing argument"
        );
        $response->getBody()->write(json_encode($error));
        return $response->withHeader('content-type', 'application/json')->withStatus(200);
    }
});

$app->run();
